/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include "nrf_drv_spi.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <string.h>
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "app_uart.h"
#include "nrf_delay.h"
#include "nrf.h"
#include "bsp.h"

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 256                         /**< UART RX buffer size. */

#define SPI_MISO 14
#define SPI_MOSI 13
#define  SPI_SCK 12
#define MAX30003_CS_PIN 11


#define WREG 0x00
#define RREG 0x01

#define   STATUS          0x01
#define   EN_INT          0x02
#define   EN_INT2         0x03
#define   MNGR_INT        0x04
#define   MNGR_DYN        0x05
#define   SW_RST          0x08
#define   SYNCH           0x09
#define   FIFO_RST        0x0A
#define   INFO            0x0F
#define   CNFG_GEN        0x10
#define   CNFG_CAL        0x12
#define   CNFG_EMUX       0x14
#define   CNFG_ECG        0x15
#define   CNFG_RTOR1      0x1D
#define   CNFG_RTOR2      0x1E
#define   ECG_FIFO_BURST  0x20
#define   ECG_FIFO        0x21
#define   RTOR            0x25
#define   NO_OP           0x7F

#define SPI_INSTANCE  1 /**< SPI instance index. */
static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);  /**< SPI instance. */
static volatile bool spi_xfer_done;  /**< Flag used to indicate that SPI instance completed the transfer. */


uint8_t       m_tx_buf[10] ;   
static uint8_t       m_rx_buf[10];   
uint8_t m_length; 

void MAX30003_Reg_Write (unsigned char WRITE_ADDRESS, unsigned long data);
void MAX30003_ReadID(void);
void max30003_sw_reset(void);
void max30003_synch(void);
void max30003_Init();

/**
 * @brief SPI user event handler.
 * @param event
 */
void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}
 void spi_event_handler(nrf_drv_spi_evt_t const * p_event,
                       void *                    p_context)
{
    spi_xfer_done = true;
    if (m_rx_buf[0] != 0)
    {
    	;
    }
}

/*static void power_manage(void)
{
	ret_code_t err_code = sd_app_evt_wait();
	APP_ERROR_CHECK(err_code);
}*/
static void uart_init()
{
    uint32_t err_code;

    const app_uart_comm_params_t comm_params =
    {
      RX_PIN_NUMBER,
      TX_PIN_NUMBER,
      RTS_PIN_NUMBER,
      CTS_PIN_NUMBER,
      APP_UART_FLOW_CONTROL_ENABLED,
      false,
      UART_BAUDRATE_BAUDRATE_Baud115200
    };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOWEST,
                         err_code);

    APP_ERROR_CHECK(err_code);

}
int main(void)
{    
     //uint32_t err_code = 4;
	// log_init();
	// NRF_LOG_WARNING("warning color testing! Code = %d",err_code);
	uart_init();

	printf("\r\nBlinky Uart \r\n");
	
    nrf_gpio_cfg (MAX30003_CS_PIN, NRF_GPIO_PIN_DIR_OUTPUT, NRF_GPIO_PIN_INPUT_DISCONNECT, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_H0S1, NRF_GPIO_PIN_NOSENSE);		  
	
    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
    //spi_config.ss_pin   = SPI_SS_PIN;
    spi_config.miso_pin = SPI_MISO;
    spi_config.mosi_pin = SPI_MOSI;
    spi_config.sck_pin  = SPI_SCK;
	spi_config.mode = NRF_DRV_SPI_MODE_0;
	spi_config.frequency = NRF_DRV_SPI_FREQ_1M;
    APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler, NULL));
	
    max30003_Init();

    while (1)
    {
		MAX30003_ReadID();
        nrf_delay_ms(100);
		//nrf_gpio_pin_set(MAX30003_CS_PIN );	
		//nrf_delay_ms(20);
    }
}


void MAX30003_Reg_Write (unsigned char WRITE_ADDRESS, unsigned long data)
{
  uint8_t wRegName = (WRITE_ADDRESS<<1) | WREG;

  m_tx_buf[0]=wRegName;
  m_tx_buf[1]=(uint8_t)(data>>16);
  m_tx_buf[2]=(uint8_t)(data>>8);
  m_tx_buf[3]=(uint8_t)(data);

    m_length=1;
  memset(m_rx_buf, 0, m_length);
  spi_xfer_done = false;

  nrf_gpio_pin_clear(MAX30003_CS_PIN );
  nrf_delay_ms(1);    
  
  APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, m_tx_buf, m_length, m_rx_buf, 0));   
  while (!spi_xfer_done){
    __WFE();
    }
 nrf_gpio_pin_set( MAX30003_CS_PIN);
 nrf_delay_ms(1);


}


void MAX30003_ReadID(void)
{
    uint8_t Reg_address=CNFG_RTOR1;//INFO;

   m_tx_buf[0] = (Reg_address<<1 ) | RREG;
   m_tx_buf[1]=0x00;
   m_tx_buf[2]=0x00;
   m_tx_buf[3]=0x00;

    m_length=4;

	spi_xfer_done = false;

   nrf_gpio_pin_clear(MAX30003_CS_PIN );	
	APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, m_tx_buf, m_length, m_rx_buf, 4)); 
	app_uart_get(&m_rx_buf);
	while (!spi_xfer_done){
		__WFE();
	}
	nrf_gpio_pin_set(MAX30003_CS_PIN );
		
}

void max30003_sw_reset(void)
{
    MAX30003_Reg_Write(SW_RST,0x000000);
    nrf_delay_ms(100);
}

void max30003_synch(void)
{
    MAX30003_Reg_Write(SYNCH,0x000000);
}



void max30003_Init()
{
	max30003_sw_reset();
    nrf_delay_ms(100);

    MAX30003_Reg_Write(CNFG_GEN, 0x080004);
    nrf_delay_ms(100);

    MAX30003_Reg_Write(CNFG_CAL, 0x720000);  // 0x700000
    nrf_delay_ms(100);

    MAX30003_Reg_Write(CNFG_EMUX,0x0B0000);
    nrf_delay_ms(100);

    MAX30003_Reg_Write(CNFG_ECG, 0x005000);  // d23 - d22 : 10 for 250sps , 00:500 sps
    nrf_delay_ms(100);

    MAX30003_Reg_Write(CNFG_RTOR1,0x3fa300);//
    nrf_delay_ms(100);

  //  MAX30003_Reg_Write(MNGR_INT, 0x000004);
    nrf_delay_ms(100);
	
	MAX30003_Reg_Write(EN_INT,0x000400);
    nrf_delay_ms(100);

    //max30003_synch();

    nrf_delay_ms(100);

}
